/**********************************************************************
$Id:$

Copyright 2001-2002, Microbrew Software

Part of Microbrew Message Center
Multi-server Mailer Suite

http://www.microbrew.org/products/mmc/

by Bulent Yilmaz <bu@microbrew.org>
and James Brown <jim@microbrew.org>

Microbrew Message Center is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

Microbrew Message Center is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along with
Microbrew Message Center; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

**********************************************************************/

#include <stdio.h>
#include <mysql/mysql.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/* Configuration file from autoconf. */
#include "../config.h"

/* Local library. */
#include "confread.h"

#define SERV_TCP_PORT 4000

#define PREFIX "/usr/local/mmc"
#define CONFFILE PREFIX "/etc/oneuser.conf"

#define ROOTBOXDIR PREFIX "/boxes"
#define VIRTFILE PREFIX "/etc/virtusertable"

#define DBCONF PREFIX "/etc/db.conf"
#define MAILSRVCONF PREFIX "/etc/mailsrv.conf"

int oneuser(int in_sock) {

    MYSQL *dbs=NULL;
    MYSQL_RES *result=NULL;
    MYSQL_ROW myrow=NULL;
    char *dotmark=NULL;
    char *mydomain=NULL, *myusername=NULL, *mypassword=NULL, *myaliases=NULL;
    char *username=NULL, *domain=NULL;
    int i=0,j=0,k=0;
    char buildquery[2048], reader[2048];
    int readsize;
    char salt[3];
    char *dbfilename=NULL;
    struct stat filestat;
    char filename[2048];
    char tfilename[2048];
    FILE *authfile;
    FILE *virtuserfile;
    char *mysqlhost, *mysqluser, *mysqlpass;
    char *myhostname, *deliveryuser, *mailboxuid, *mailboxgid;
    int uid_i, gid_i;

    if((mysqlhost = readconf(DBCONF, "DBSERVER")) == NULL) {
        printf("Could not read 'DBSERVER' from %s.\n", DBCONF);
        return 1;
    }
                            
    if((mysqluser = readconf(DBCONF, "USERNAME")) == NULL) {
        printf("Could not read 'USERNAME' from %s.\n", DBCONF);
        return 2;
    }
                                                        
    if((mysqlpass = readconf(DBCONF, "PASSWORD")) == NULL) {
        printf("Could not read 'PASSWORD' from %s.\n", DBCONF);
        return 3;
    }
    
    if((myhostname = readconf(MAILSRVCONF, "HOSTNAME")) == NULL) {
        printf("Could not read 'HOSTNAME' from %s.\n", MAILSRVCONF);
        return 4;
    }
    
    if((deliveryuser = readconf(MAILSRVCONF, "DELIVERY")) == NULL) {
        printf("Could not read 'DELIVERY' from %s.\n", MAILSRVCONF);
        return 5;
    }
    
    if((mailboxuid = readconf(MAILSRVCONF, "MAILBOXUID")) == NULL) {
        printf("Could not read 'MAILBOXUID' from %s.\n", MAILSRVCONF);
        return 6;
    }
    
    uid_i = atoi(mailboxuid);
    
    if((mailboxgid = readconf(MAILSRVCONF, "MAILBOXGID")) == NULL) {
        printf("Could not read 'MAILBOXGID' from %s.\n", MAILSRVCONF);
        return 7;
    }
    
    gid_i = atoi(mailboxgid);
    
    if((readsize=read(in_sock, reader, 2047)) > 0) {
        close(in_sock);
        reader[readsize] = '\0';
        myusername=strtok(reader, " \n\r");
        if(myusername != NULL) {
            mydomain=strtok(NULL, " \n\r");
            if(mydomain != NULL) {
                username=strdup(myusername);
                domain=strdup(mydomain);
            } else return 1;
        } else return 1;
    } else {
        close(in_sock);
        return 1;
    }
    
    printf("User: %s, Domain: %s\n", username, domain);
    
    if((dbs = mysql_init(NULL))!=NULL) {
        if(mysql_real_connect(dbs, mysqlhost, mysqluser, mysqlpass, NULL, 0, NULL, 0)) {
            if(!mysql_select_db(dbs, "mmccust")) {
            
                mydomain = strdup(domain);
    
                while( (dotmark = strchr(mydomain, '.')) != NULL ) {
                    *dotmark = '_';
                }
            
                snprintf(buildquery, 2047, "SELECT username, password, aliases FROM %s_users WHERE username='%s'", mydomain, username);
            
                free(mydomain);
            
                if(!mysql_query(dbs, buildquery)) {
                    result = mysql_use_result(dbs);
                    
                    if(myrow = mysql_fetch_row(result)) {
                        /* User and domain exist, so proceed. */
                        
                        mypassword=strdup(myrow[1]);
                        myaliases=strdup(myrow[2]);
                        
                        /* First clear out the buffer for the database read. */
                        
                        while((myrow = mysql_fetch_row(result)) != NULL);
                        
                        /* Now check for the directories. */
                        
                        snprintf(filename, 2047, "%s/%s", ROOTBOXDIR, domain);
                        
                        if(!stat(filename, &filestat)) {
                            printf("'%s' already exists.  Checking subdirs.\n",filename);
                            
                            snprintf(filename, 2047, "%s/%s/auth", ROOTBOXDIR, domain);
                            if(!stat(filename, &filestat)) {
                                printf("'%s' already exists.\n",filename);
                            } else {
                                if(mkdir(filename, 0x01FF)) {  /* ~umask & o777 */
                                    perror("mkdir auth");
                                    return 1;
                                } else printf("'%s' created.\n",filename);
                            }
                            
                            snprintf(filename, 2047, "%s/%s/auth", ROOTBOXDIR, domain);
                            if(!stat(filename, &filestat)) {
                                printf("'%s' already exists.\n",filename);
                            } else {
                                if(mkdir(filename, 0x01FF)) {  /* ~umask & o777 */
                                    perror("mkdir auth");
                                    return 1;
                                } else printf("'%s' created.\n",filename);
                            }
                            
                        } else {
                            printf("'%s' does not exist.  Creating.\n",filename);
                            
                            /* First check to make sure that the entry exists in the database. */
                            
                            if(!mysql_select_db(dbs, "mmcadmin")) {
                                snprintf(buildquery, 2047, "SELECT pdomain, sdomains FROM customers WHERE pdomain='%s'", domain);
                                
                                if(!mysql_query(dbs, buildquery)) {
                                    result = mysql_use_result(dbs);
                                    
                                    if((myrow = mysql_fetch_row(result)) != NULL) {
                                        if(strlen(myrow[1])) {
                                            /* Secondary domains to add to virtusertable. */
                                            
                                            mydomain = strtok(myrow[1], "\r\n, ");
                                            
                                            do {
                                                snprintf(tfilename, 2047, "%s -u hash /etc/mail/virtusertable @%s \%1@%s", EDITMAP_BIN, mydomain, domain);
                                                system(tfilename);
                                            } while((mydomain = strtok(NULL, "\r\n, ")) != NULL);
                                
                                        }
                                    } else {
                                        puts("Could not find domain in database.  Probably a data sync problem.");
                                        return 1;
                                    }
                                } else {
                                    puts("Could not query database");
                                    return 1;
                                } 
                            } else {
                                puts("Could not open database 'mmcadmin'");
                                return 1;
                            }
                                                        
                            if(mkdir(filename, 0x01FF)) {  /* ~umask & o777 */
                                perror("mkdir base");
                                return 1;
                            } else {
                                snprintf(filename, 2047, "%s/%s/auth", ROOTBOXDIR, domain);
                                if(mkdir(filename, 0x01FF)) {  /* ~umask & o777 */
                                    perror("mkdir auth");
                                    return 1;
                                } else {
                                    chown(filename, uid_i, gid_i);
                                    
                                    snprintf(filename, 2047, "%s/%s/mail", ROOTBOXDIR, domain);
                                    if(mkdir(filename, 0x01FF)) {
                                        perror("mkdir mail");
                                        return 1;
                                    } else chown(filename, uid_i, gid_i);
                                }
                            }
                        }
                        
                        snprintf(filename, 2048, "%s/%s/auth/%s", ROOTBOXDIR, domain, username);
                        
                        if((authfile = fopen(filename, "w+")) != NULL) {
                        
                            salt[0]=(rand() % 26) + 65;
                            salt[1]=(rand() % 26) + 97;
                            salt[2]=0;
                        
                            fprintf(authfile, "%s:%s:%s:%s:::\n", username, crypt(mypassword, &salt[0]), mailboxuid, mailboxgid);
                            fclose(authfile);
                            chown(filename, uid_i, gid_i);
                            
                            /* Also create/chown the mailbox itself so that LDA can deliver. */
                            
                            snprintf(filename, 2048, "%s/%s/mail/%s", ROOTBOXDIR, domain, username);
                            if(!stat(filename, &filestat)) {
                                /* File exists, check ownership. */
                                
                                if(filestat.st_uid != uid_i) chown(filename, uid_i, gid_i);
                                else if(filestat.st_gid != gid_i) chown(filename, uid_i, gid_i);
                            } else {
                                /* File does not exist.  Create. */
                                
                                if((authfile = fopen(filename, "w+")) != NULL) {
                                    fclose(authfile);
                                    chown(filename, uid_i, gid_i);
                                } else printf("Could not open mailbox file '%s'\n",filename);
                            }                        
                        }
                        
                        /* Add entries to virtusertable */
                        
                        if(strlen(myaliases)) {
                            mydomain = strtok(myaliases, "\r\n, ");
                            do {
                                snprintf(filename, 2047, "%s -u hash /etc/mail/virtusertable %s@%s %s@%s", EDITMAP_BIN, mydomain, domain, username, domain);
                                system(filename);
                            } while ( (mydomain=strtok(NULL, "\r\n, ")) != NULL );
                        }
                        
                        /* The actual delivery line. */
                        
                        snprintf(filename, 2047, "%s -u hash /etc/mail/virtusertable %s@%s mmclda:%s@%s", EDITMAP_BIN, username, domain, username, domain);
			system(filename);
                    
                        /* Do a couple free()s and we're done. */
                    
                        free(mypassword);
                        free(myaliases);
                    
                    } else {
                        puts("Could not find username");
                        
                        /* Delete from virtusertable */
                        snprintf(filename, 2047, "%s -x hash /etc/mail/virtusertable %s@%s", EDITMAP_BIN, username, domain);
                        system(filename);
                    }
                } else puts("Customer lookup failed.");
            } else puts("Could not select the 'mmcadmin' database.");
        } else puts("Could not connect to the database.");
    
        mysql_close(dbs);    
    } else puts("Could not allocate memory for connect.");

    return 0;

}

long *allowedips=NULL;
int numallowedips=0;

int loadconf() {

    /* Returns 0 upon load, returns non-zero upon failure. */
    
    FILE *conffile=NULL;
    char confline[256];
    char *ip, *action;
    struct in_addr byteaddr;
    int lineno=0;
    long *saver=NULL;
    
    conffile=fopen(CONFFILE, "r");
    
    if(conffile) {
        do {
            fgets(confline, 255, conffile);
            lineno++;
            
            if(confline[0] != '#') {
                ip=strtok(confline, " \t\r\n#");
                action=strtok(NULL, " \t\r\n#");
                
                if( (ip!=NULL) && (action!=NULL) ) {
                    if(inet_aton(ip, &byteaddr)) {
                        if(!strncmp(action, "ALLOW", 6)) {
                            numallowedips++;
                            saver=allowedips;
                            if(( allowedips = (long *)realloc(allowedips, (numallowedips)*sizeof(long)) ) != NULL) {
                                allowedips[numallowedips-1]=byteaddr.s_addr;
                            } else {
                                perror("loadconf: realloc() failed");
                                if(saver!=NULL) free(saver);
                                return 1;
                            }
                        } else if(!strncmp(action, "RELAY", 6)) {
                            puts("RELAY not currently supported.");
                        
                        } else printf("Warn: Invalid keyword in '%s' on line %d.\n", CONFFILE, lineno);
                    } else printf("Warn: Invalid IP in '%s' on line %d.\n", CONFFILE, lineno);
                } else printf("Warn: Invalid format in '%s' on line %d.\n", CONFFILE, lineno);
            }
        } while(!feof(conffile));
        
        fclose(conffile);
    } else return 1;
    
    return 0;
}

int main(int argc, char *argv[]) {

    int sockfd, newsockfd;
    struct sockaddr_in cli_addr, serv_addr, conn_addr;
    int clilen;
    int childpid;
    int found, i;
    
    loadconf();

    if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        puts("Couldn't open socket for listening.");
        return 1;
    }
    
    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
    serv_addr.sin_port = htons(SERV_TCP_PORT);
    
    if (bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) {
        puts("Couldn't bind to socket.");
        return 1;
        puts("I should have exited but I didn't.");
    }

    listen(sockfd, 5);
    
    while(1) {
#if TESTOUTPUT    
        printf("Waiting for connection.\n");
        fflush(stdout);
#endif
        clilen = sizeof(cli_addr);
        newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, &clilen);
        
        if (newsockfd < 0) {
            puts("Server accept error");
        } else {
            found=0;
            i=0;
            if(!getpeername(newsockfd, (struct sockaddr *) &conn_addr, &clilen)) {
            
                while((i<numallowedips)&&(!found)) {
                    if(conn_addr.sin_addr.s_addr==allowedips[i]) found=1;
                    i++;
                }
                
                if(found) {
                    childpid=fork();
                    if(childpid == 0) {
                        /* I am the child. */
                        close(sockfd);
                        oneuser(newsockfd);
                        return 0;
                    } else {
                        /* I am the parent. */
                        
                        close(newsockfd);
                    }
                } else {
                    close(newsockfd);
                    printf("IP %x is not allowed to connect.\n", conn_addr.sin_addr.s_addr);
                }
            } else {
                close(newsockfd);
                puts("Could not determine remote side IP.");
            }
        }
    }

    return 0;

}
