/**********************************************************************
$Id:$

Copyright 2001-2002, Microbrew Software

Part of Microbrew Message Center
Multi-server Mailer Suite

http://www.microbrew.org/products/mmc/

by Bulent Yilmaz <bu@microbrew.org>
and James Brown <jim@microbrew.org>

Microbrew Message Center is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

Microbrew Message Center is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along with
Microbrew Message Center; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

**********************************************************************/

#include <stdio.h>
#include <dirent.h>
#include <stdio.h>
#include <mysql/mysql.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

/* Configuration file from autoconf. */
#include "../config.h"

/* Local library */
#include "confread.h"

#define BASEDIR PREFIX "/boxes"
#define DBCONF PREFIX "/etc/db.conf"

int deleteallrecursive(char *directory) {
/*
    int deleteallrecursive(char *directory);
    
    Deletes directory pointed to by directory recursively by deleting
    contents first.  Returns 0 upon success, returns -1 on failure.
*/    

    struct stat filestat;
    int files, i;
    struct dirent **filelist=NULL;
    char myfilename[1024];
    
    files = scandir(directory, &filelist, 0, alphasort);
    
    if(files < 0) {
        perror("deleteallrecursive: scandir");

        return -1;        
    } else {
    
        for(i=0; i < files; i++) {
            if(filelist[i]->d_name[0] != '.') {
                snprintf(myfilename, 1023, "%s/%s", directory, filelist[i]->d_name);
                if(!stat(myfilename, &filestat)) {
                    if((filestat.st_mode & S_IFDIR)) {
                        if(deleteallrecursive(myfilename)) {
                            /* An error occurred.  Drop out. */
                            puts("deleteallrecursive: recursion failure.");
                            
                            return -1;
                        }
                    } else {
                        if(unlink(myfilename)) perror("deleteallrecursive: unlink");
                    }
                } else {
                    perror("delteallrecursive: stat");
                }
            }
            free(filelist[i]);
        }
        free(filelist);
    }
    
    if(!rmdir(directory)) return 0;
    else {
        perror("deleteallrecursive: rmdir");
        
        return -1;
    }

    // Just in case it drops through, return an error:
    return -1;    
}

int main() {

    struct dirent **toplevel=NULL, **userlist=NULL;
    char *subdir=NULL, *deletename=NULL;
    int domains, users;
    int i,j;
    char querystring[2048];
    char *custdbname=NULL, *dotmark=NULL;
    
    MYSQL *dbs=NULL;
    MYSQL_RES *result=NULL;
    MYSQL_ROW myrow=NULL;
    
    char *mysqlhost=NULL, *mysqluser=NULL, *mysqlpass=NULL;
    
    if((mysqlhost = readconf(DBCONF, "DBSERVER")) == NULL) {
        printf("Could not read 'DBSERVER' from %s.\n", DBCONF);
        return 1;
    }
    
    if((mysqluser = readconf(DBCONF, "USERNAME")) == NULL) {
        printf("Could not read 'USERNAME' from %s.\n", DBCONF);
        return 2;
    }
    
    if((mysqlpass = readconf(DBCONF, "PASSWORD")) == NULL) {
        printf("Could not read 'PASSWORD' from %s.\n", DBCONF);
        return 3;
    }
    
    if((dbs = mysql_init(NULL))!=NULL) {
        if(mysql_real_connect(dbs, mysqlhost, mysqluser, mysqlpass, NULL, 0, NULL, 0)) {

            domains = scandir(BASEDIR, &toplevel, 0, alphasort);
            
            if(domains < 0) {
                perror("Domain read");
                return 6;
            } else {
                for(i=0; i<domains; i++) {
                    printf("%s... ", toplevel[i]->d_name);
                    
                    if(toplevel[i]->d_name[0] != '.') {
                    
                        if(!mysql_select_db(dbs, "mmcadmin")) {
                            
                            snprintf(querystring, 2047, "SELECT pdomain FROM customers WHERE pdomain='%s'", toplevel[i]->d_name);
                            if(!mysql_query(dbs, querystring)) {
                            
                                result = mysql_use_result(dbs);
                                if(myrow = mysql_fetch_row(result)) {
                                
                                    while(myrow = mysql_fetch_row(result));
                                
                                    custdbname=strdup(toplevel[i]->d_name);
                                    
                                    while( (dotmark = strchr(custdbname, '.')) != NULL ) {
                                        *dotmark = '_';
                                    }
                                    
                                    /* Delete extraneous data from mail directory */
                                    
                                    subdir=malloc(strlen(BASEDIR) + strlen(toplevel[i]->d_name) + 7);
                                    snprintf(subdir, strlen(BASEDIR) + strlen(toplevel[i]->d_name) + 7, "%s/%s/mail", BASEDIR, toplevel[i]->d_name);
                                    printf("  %s\n", subdir);
                                    
                                    users = scandir(subdir, &userlist, 0, alphasort);
                                    
                                    if(users < 0) {
                                        perror("Mailbox read");
                                        return 7;
                                    } else {
                                        for(j=0; j<users; j++) {
                                        
                                            if(userlist[j]->d_name[0] != '.') {
                                                printf("    %s... ", userlist[j]->d_name);
                                                
                                                if(!mysql_select_db(dbs, "mmccust")) {
                                                
                                                    snprintf(querystring, 2047, "SELECT username FROM %s_users WHERE username='%s'", custdbname, userlist[j]->d_name);
                                                    if(!mysql_query(dbs, querystring)) {
                                                      
                                                        result = mysql_use_result(dbs);
                                                        if(myrow = mysql_fetch_row(result)) {
                                                            while(myrow = mysql_fetch_row(result));
                                                            puts("User found.");
                                                        } else {
                                                            printf("Deleting user... ");
                                                            
                                                            /* Delete User */
                                                            deletename=malloc(strlen(BASEDIR) + strlen(toplevel[i]->d_name) + strlen(userlist[j]->d_name) + 8);
                                                            snprintf(deletename, strlen(BASEDIR) + strlen(toplevel[i]->d_name) + strlen(userlist[j]->d_name) + 8, "%s/%s/mail/%s", BASEDIR, toplevel[i]->d_name, userlist[j]->d_name );
                                                            
                                                            if(!unlink(deletename)) puts("Successful.");
                                                            else puts("Delete failed!");
                                                        }
                                                    } else puts("Could not perform query");
                                                } else puts("Could not select database mmccust");
                                            }    
                                            free(userlist[j]);
                                        }
                                        free(userlist);
                                    }
                                    
                                    free(subdir);
                                    
                                    /* Now, do the exact same for the auth directory */
                                    
                                    subdir=malloc(strlen(BASEDIR) + strlen(toplevel[i]->d_name) + 7);
                                    snprintf(subdir, strlen(BASEDIR) + strlen(toplevel[i]->d_name) + 7, "%s/%s/auth", BASEDIR, toplevel[i]->d_name);
                                    printf("  %s\n", subdir);
                                    
                                    users = scandir(subdir, &userlist, 0, alphasort);
                                    
                                    if(users < 0) {
                                        perror("Authdir read");
                                        return 8;
                                    } else {
                                        for(j=0; j<users; j++) {
                                        
                                            if(userlist[j]->d_name[0] != '.') {
                                                printf("    %s... ", userlist[j]->d_name);
                                                
                                                if(!mysql_select_db(dbs, "mmccust")) {
                                                
                                                    snprintf(querystring, 2047, "SELECT username FROM %s_users WHERE username='%s'", custdbname, userlist[j]->d_name);
                                                    if(!mysql_query(dbs, querystring)) {
                                                      
                                                        result = mysql_use_result(dbs);
                                                        if(myrow = mysql_fetch_row(result)) {
                                                            while(myrow = mysql_fetch_row(result));
                                                            puts("User found.");
                                                        } else {
                                                            printf("Deleting authfile... ");
                                                            
                                                            /* Delete User */
                                                            deletename=malloc(strlen(BASEDIR) + strlen(toplevel[i]->d_name) + strlen(userlist[j]->d_name) + 8);
                                                            snprintf(deletename, strlen(BASEDIR) + strlen(toplevel[i]->d_name) + strlen(userlist[j]->d_name) + 8, "%s/%s/auth/%s", BASEDIR, toplevel[i]->d_name, userlist[j]->d_name );
                                                            
                                                            if(!unlink(deletename)) puts("Successful.");
                                                            else puts("Delete failed!");
                                                        }
                                                    } else puts("Could not perform query");
                                                } else puts("Could not select database mmccust");
                                            }    
                                            free(userlist[j]);
                                        }
                                        free(userlist);
                                    }
                                    
                                    free(subdir);
                                    
                                    free(custdbname);
                                    
                                } else {
                                    printf("Deleting... ");
                                    
                                    /* Delete directory... */
                                    
                                    deletename=malloc(strlen(BASEDIR) + strlen(toplevel[i]->d_name) + 2);
                                    snprintf(deletename, strlen(BASEDIR) + strlen(toplevel[i]->d_name) + 2, "%s/%s", BASEDIR, toplevel[i]->d_name );
                                    
                                    if(!deleteallrecursive(deletename)) puts("Successful.");
                                    else puts("Delete failed!");
                                }
                            } else puts("Query failed");
                        } else puts("Could not select 'mmcadmin' database.");
                    }
                    free(toplevel[i]);
                }
                free(toplevel);
            }
        } else {
            puts("Could not open database.");
            return 5;
        }
    } else {
        puts("Could not allocate space for database connection.");
        return 4;
    }

    return 0;
}
